@@ -1,5 +1,5 @@
 Gammu Zarządzanie wszystkimi telefonami
-========================================
+=======================================
 
 Gammu jest biblioteką i linią komend narzędzia dla telefonów
 komórkowych. Jest udostępniany na licencji GNU GPL w wersji 2.
@@ -11,9 +11,18 @@ był oparty na Gnokii <http://www.gnokii.org> i później mygnokii
 Obecnie projekt jest prowadzony przez Michal Cihar <michal@cihar.com> z
 pomocą wielu użytkowników.
 
+.. image:: https://travis-ci.org/gammu/gammu.svg?branch=master
+    :alt: Build status :target: https://travis-ci.org/gammu/gammu
+
+.. image:: https://hosted.weblate.org/widgets/gammu-status-badge.png
+    :alt: Translation status :target:
+    https://hosted.weblate.org/engage/gammu/?utm_source=widget
+
+.. image:: https://scan.coverity.com/projects/2890/badge.svg?flat=1
+    :alt: Coverity scan :target: https://scan.coverity.com/projects/2890
 
 Więcej informacji
-==================
+=================
 
 Możesz znaleźć więcej informacji na temat <http://wammu.eu/gammu/>.
 
@@ -23,13 +32,13 @@ wersję HTML za pomocą zrobić manual-html, który jest na:
 
 
 Opinie i raporty o błędach
-============================
+==========================
 
 Wszelkie uwagi są mile widziane, patrz <http://wammu.eu/support/> po
 informację jak skontaktować się z programistami.
 
 
 Wsparcie programistów
-======================
+=====================
 
 Można docenić pracę programistów na <http://wammu.eu/donate/>.
