@@ -184,7 +184,7 @@ GSM_Error GSM_SetFeatureString(GSM_Feature *list, const char *string)
 		}
 		out++;
 		if (out >= GSM_MAX_PHONE_FEATURES) {
-			smfprintf(NULL, "Too much features: %s\n", pos);
+			smfprintf(NULL, "Too many features: %s\n", pos);
 			error = ERR_MOREMEMORY;
 			break;
 		}
@@ -958,10 +958,12 @@ GSM_PhoneModel allmodels[] = {
 	{"E1550",	"E1550",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, F_RESET_AFTER_TIMEOUT, F_HUAWEI_INIT, 0}},
 	{"E1750",	"E1750",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, F_RESET_AFTER_TIMEOUT, 0}},
 	{"E1752",	"E1752",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, F_RESET_AFTER_TIMEOUT, 0}},
+	{"E303",	"E303",		"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, F_RESET_AFTER_TIMEOUT, F_HUAWEI_INIT, 0}},
 	{"EM770W",	"EM770W",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, F_RESET_AFTER_TIMEOUT, F_HUAWEI_INIT, 0}},
 	{"E3372",	"E3372",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, F_RESET_AFTER_TIMEOUT, F_HUAWEI_INIT, 0}},
 	{"E3276",	"E3276",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, F_RESET_AFTER_TIMEOUT, F_HUAWEI_INIT, 0}},
 	{"K3765",	"K3765",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, F_RESET_AFTER_TIMEOUT, F_HUAWEI_INIT, 0}},
+	{"K4511",	"K4511",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, F_RESET_AFTER_TIMEOUT, F_HUAWEI_INIT, 0}},
 
 	/* ZTE */
 	{"MF190",	"MF190",	"",		{F_ZTE_INIT, 0}},
@@ -969,6 +971,9 @@ GSM_PhoneModel allmodels[] = {
 	/* Ubinetics */
 	{"GDC201",	"GDC201",	"",		{F_SMS_NO_ME, 0}},
 
+	/* Other */
+	{"SIMCOM_SIM300D","SIMCOM_SIM300D","",		{F_ENCODED_USSD, 0}},
+
 	/* Qualcomm */
 	{"HP un2420 Mobile Broadband Module", "un2420", "", {F_NO_STOP_CUSD, 0}},
 
