@@ -201,6 +201,8 @@ void SMSD_Terminate(GSM_SMSDConfig *Config, const char *msg, GSM_Error error, gb
 		SMSD_LogError(DEBUG_ERROR, Config, msg, error);
 	} else if (rc != 0) {
 		SMSD_LogErrno(Config, msg);
+	} else {
+		SMSD_LogError(DEBUG_INFO, Config, msg, error);
 	}
 
 	/* Disconnect from phone */
@@ -761,6 +763,9 @@ GSM_Error SMSD_ReadConfig(const char *filename, GSM_SMSDConfig *Config, gboolean
 	GSM_SetConfigNum(Config->gsm, 1);
 	gammucfg->UseGlobalDebugFile = FALSE;
 
+	/* Gammu library wide logging to our log */
+	GSM_SetDebugFunction(SMSD_Log_Function, Config, GSM_GetGlobalDebug());
+
 	/* Force debug level in Gammu */
 	if ((DEBUG_GAMMU & Config->debug_level) != 0) {
 		strcpy(gammucfg->DebugLevel, "textall");
@@ -1940,11 +1945,11 @@ GSM_Error SMSD_MainLoop(GSM_SMSDConfig *Config, gboolean exit_on_failure, int ma
 			/* Did we reach limit for errors? */
 			if (max_failures != 0 && initerrors > max_failures) {
 				Config->failure = ERR_TIMEOUT;
-				SMSD_Log(DEBUG_INFO, Config, "Reached maximal number of failures (%d), terminating", max_failures);
+				SMSD_Log(DEBUG_INFO, Config, "Reached maximum number of failures (%d), terminating", max_failures);
 				break;
 			}
 			if (initerrors++ > 3) {
-				SMSD_Log(DEBUG_INFO, Config, "Going to 30 seconds sleep because of too much connection errors");
+				SMSD_Log(DEBUG_INFO, Config, "Going to 30 seconds sleep because of too many connection errors");
 
 				for (i = 0; i < 60; i++) {
 					if (Config->shutdown)
@@ -1972,6 +1977,9 @@ GSM_Error SMSD_MainLoop(GSM_SMSDConfig *Config, gboolean exit_on_failure, int ma
 					GSM_SetIncomingCall(Config->gsm, TRUE);
 				}
 
+				/* We use polling so store messages to SIM */
+				GSM_SetIncomingSMS(Config->gsm, TRUE);
+
 				GSM_SetSendSMSStatusCallback(Config->gsm, SMSD_SendSMSStatusCallback, Config);
 				/* On first start we need to initialize some variables */
 				if (first_start) {
